package tutusinternal

// Syscall0 performs syscall with 0 arguments.
func Syscall0(name string) any {
	return nil
}

// Syscall0NoReturn performs syscall with 0 arguments.
func Syscall0NoReturn(name string) {
}

// Syscall1 performs syscall with 1 arguments.
func Syscall1(name string, arg any) any {
	return nil
}

// Syscall1NoReturn performs syscall with 1 arguments.
func Syscall1NoReturn(name string, arg any) {
}

// Syscall2 performs syscall with 2 arguments.
func Syscall2(name string, arg1, arg2 any) any {
	return nil
}

// Syscall2NoReturn performs syscall with 2 arguments.
func Syscall2NoReturn(name string, arg1, arg2 any) {
}

// Syscall3 performs syscall with 3 arguments.
func Syscall3(name string, arg1, arg2, arg3 any) any {
	return nil
}

// Syscall3NoReturn performs syscall with 3 arguments.
func Syscall3NoReturn(name string, arg1, arg2, arg3 any) {
}

// Syscall4 performs syscall with 4 arguments.
func Syscall4(name string, arg1, arg2, arg3, arg4 any) any {
	return nil
}

// Syscall4NoReturn performs syscall with 4 arguments.
func Syscall4NoReturn(name string, arg1, arg2, arg3, arg4 any) {
}
